# Copyright (C) 2025-2026 AKIOUD AI, SAS <contact@akioud.ai>
# SPDX-License-Identifier: GPL-3.0-or-later

"""Shared utility helpers for AKIOS core."""

from .network import check_network_available

__all__ = ["check_network_available"]
