# Copyright (C) 2025-2026 AKIOUD AI, SAS <contact@akioud.ai>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

"""
AKIOS Analytics Module

Provides cost tracking, budget visualization, and usage analytics.
"""

from .cost_tracker import (
    get_budget_status,
    calculate_budget_percentage,
    get_cost_color,
    get_cost_status_icon,
    format_cost_breakdown,
)

__all__ = [
    'get_budget_status',
    'calculate_budget_percentage',
    'get_cost_color',
    'get_cost_status_icon',
    'format_cost_breakdown',
]
