# Copyright (C) 2025-2026 AKIOUD AI, SAS <contact@akioud.ai>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

"""
AKIOS REST API — ``akios serve``

Provides a lightweight HTTP interface to the AKIOS runtime.
Requires the ``api`` optional dependency group::

    pip install akios[api]
"""
